//modules
var express = require('express');
var favicon = require('serve-favicon');
var cookieParser = require('cookie-parser');
var bodyParser = require('body-parser');

var path = require('path');

// properties
var properties = require('./lib/configurationInterpreter');

// logger
var logLib = require("./lib/logger");

// routes
var routes = require('./routes/index');
var print = require('./routes/print');

var app = express();

app.set('log', logLib.logger('labelprint'));

var log = app.get('log');

// view engine setup
app.engine('html', require('ejs').renderFile);
app.set('view engine', 'html');

app.use(bodyParser.json({
	limit : '1000kb'
}));
app.use(bodyParser.urlencoded({
	extended : false
}));
app.use(cookieParser());
app.use(express.static(path.join(__dirname, 'public')));
app.use("/", routes);

// REST - Methods
// posted data will be displayed on selection page and only selected data will
// be printed
app.post('/printSelected', print.showSelectionPage);
// demo
app.get('/printSelected', print.showSelectionDemoPage);
// posted data will be printed silently
app.post('/printReceipt', print.instantPrint);
app.post('/printArticles', print.printArticles);
app.post('/printAcumulated', print.acumulate);

// catch 404 and forward to error handler
app.use(function(req, res, next) {
	var err = new Error('Not Found');
	err.status = 404;
	next(err);
});

// error handlers

// development error handler
// will print stacktrace
if (app.get('env') === 'development') {
	app.use(function(err, req, res, next) {
		res.status(err.status || 500);
		res.render('error', {
			message : err.message,
			error : err
		});
	});
}

// production error handler
// no stacktraces leaked to user
app.use(function(err, req, res, next) {
	res.status(err.status || 500);
	res.render('error', {
		message : err.message,
		error : {}
	});
});

// starting server with properties
properties.loadConfiguration(function(props) {
	log.info('printlabel listens to ' + props.port);
	app.listen(props.port);
});

module.exports = app;
